/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.04.30.
*   Chip         -  AVR
*
*   kprint - szamokat es szoveget kiiro objektum 
*
*   Igazabol ez egy absztrekt osztaly, lcd-hez, soros port-hoz... stb.
*   A belole szarmaztatott osztalyban kell egy writechar() fuggvenyt
*   definialni,  ami elvegzi egy betu kiirasat.
*   A szamokat alap esetben elojelesen, decimalisan irja ki.
*   Opcionalis formatumok:
*   X     -  hexadecimalis int              -  0x0000
*   x     -  hexadecimalis char             -  0x00
*   b     -  binaris char                   -  0B11110000
*   1..5  -  fix hosszusagu decimalis       -  1-5 szamjegy + elojel, ha van
*   0     -  valtozu hosszusagu decimalis
*   
*   Ha a szam nem fer ki a megadott formatumban, *****-kat ir ki.
*   
*******************************************************************************/

#ifndef tkiraaly_kprint
#define tkiraaly_kprint




#include <avr/pgmspace.h>
#include "tkiraaly_avr_macros.h"




#define F( s)    (const khelper *)( PSTR( s))    // makro khelper-hez
class khelper;                                   // seged tipus PSTR nyomtatashoz




class kprint
{
   public:
      kprint() {};
      virtual void kputchar( U8) {};             // egy betut ir ki
      void print( char n, U8= '0');              // format - b, x, 0..3
      void print( U8 n,   U8= '0');
      void print( int n,  U8= '0');              // format - X, 0..5
      void print( U16 n,  U8= '0');
      void print( const char *);
      void print( const khelper *);
};




void kprint::print(  const char *s)              // string kiirasa RAM-bol
{
   while( *s) kputchar( *s++); 
}




void kprint::print( const khelper *s)            // string kiirasa FLASH-bol
{
    U8 c;
    PGM_P p = ( PGM_P)( s);
    while (1) {
       c = pgm_read_byte( p++);                  // ez valojaban makro
       if (c == 0) break;
       kputchar( c);
  }
}




void kprint::print( char data, U8 format)        // char kiirasa - b, x, 0..3 
{
   print( (int)data, format);
}




void kprint::print( U8 data, U8 format)          // unsigned char kiirasa - b, x, 0..3
{
   print( ( U16)data, format);
}




void kprint::print( int data, U8 format)         // int kiirasa - X, 0..5
{
   U8 e= ' ';                                    // elojel
   if( data < 0)                                 // minusz
   {
      data= -data;
      e= '-';
   }
   kputchar( e);
   print( ( U16)data, format);
}




void kprint::print( U16 data, U8 format)         // unsigned int kiirasa - X, 0..5
{
   char buf[ 9];                                 // betuknek + \0         
   char * p= buf;                                // buffer eleje
   U8 a=      10;                                // alap
   U8 h=       0;                                // hossz
   U8 t=     '0';                                // feltolto betu
   U8 i;
   switch( format)
   {
      case 'b': a=  2; h= 8; break;              // 0B00001111
      case 'x': a= 16; h= 2; break;              // 0x00
      case 'X': a= 16; h= 4; break;              // 0x0000
      case '0': h= 5; break;                     // 00000
      case '1': 
      case '2': 
      case '3': 
      case '4': 
      case '5': h= format- '0'; t= ' ';          // 0..00000
   }
   *p= '\0';                                     // \0
   i= h;
   while( i--)
   {
      if ( data == 0)                            // feltoltes
      {
         if ( format != '0') *++p= t;
      }
      else                                       // szamjegy kiszamolasa
      {
         *++p= data% a;                          // a maradek a helyiertek
         if( *p > 9) *p+= 'A'- 10;               // 9-nel nagyobb, akkor ABC..
         else        *p+= '0';                   // 9-ig szamjegy
         data/= a;
      }
   }
   if ( data > 0)                                // hiba, nem fert ki ***** 
   {
      p= buf;
      i= h;
      while( i--) *++p= '*';
   }
   switch ( format)
   {
      case 'b': kputchar( '0'); kputchar( 'B'); break;   // 0B
      case 'x':
      case 'X': kputchar( '0'); kputchar( 'x'); break;   // 0x
   }
   while( *p) kputchar( *p--);                           // kiras visszafele
}




#endif
