/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.04.30.
*   Chip         -  AVR
*
*   kserial - soros portra kiiro objektum 
*
*   Ez a kprint-bol szarmaztatott osztaly.
*   Alkalmazhato az Atmega csaladban, tipustol fuggoen, 0..3 soros portra
*   lehet az objektumot letrehozni.
*   A soros portot a korabbi makroimmal kell inicializalni, pl.:
*   USART_SET_9600_8N1
*   
*******************************************************************************/

#ifndef tkiraaly_kserial
#define tkiraaly_kserial




#include "tkiraaly_kprint.cpp"
#include "tkiraaly_avr_macros.h"




class kserial : public kprint
{
   public:
      U8 _serial_port;
      kserial( U8 s) { _serial_port= s;};        // 0, 1, 2, 3, 8
      virtual void kputchar( U8);
      void newline( void);
};




void kserial::kputchar( U8 c)                    // egy betu kiirasa
{
   switch ( _serial_port)
   {
      #ifdef UCSRB                               // ATmega8 eseten
      case 0:
         while( !B_R( UCSRA, UDRE));
         UDR= c;
         break;
      #endif
      #ifdef UCSR0B                              // csak az lesz definialva, amelyik az adott uC-ben van
      case 0:
         while( !B_R( UCSR0A, UDRE0));           // 1, ha kesz az adat fogadasara
         UDR0= c;
         break;
      #endif
      #ifdef UCSR1B
      case 1:
         while( !B_R( UCSR1A, UDRE1));
         UDR1= c;
         break;
      #endif
      #ifdef UCSR2B
      case 2:
         while( !B_R( UCSR2A, UDRE2));
         UDR2= c;
         break;
      #endif
      #ifdef UCSR3B
      case 0:
         while( !B_R( UCSR3A, UDRE3));
         UDR3= c;
         break;
      #endif
   } 
}




void kserial::newline( void)                     // kocsi vissza, soremeles
{
   kputchar( '\r');
   kputchar( '\n');
}




#endif
